@@ -8,7 +8,7 @@ from pydantic import BaseConfig, BaseModel, validator
 from sanic.response import html
 from sanic_jinja2 import SanicJinja2
 
-__version__ = "0.0.10"
+__version__ = "0.1.0"
 
 
 loader = FileSystemLoader(
@@ -21,8 +21,9 @@ def render_with_updated_context(
 ):
     context["admin_panel_title"] = cfg.name
     context["objects"] = cfg.models
-    context["url_prefix"] = cfg.URL_PREFIX
+    context["url_prefix"] = cfg.route
     context["admin_panel_version"] = __version__
+    context["round_number"] = cfg.round_number
     return html(
         self.render_string(template, request, **context),
         status=status,
@@ -32,7 +33,6 @@ def render_with_updated_context(
 
 SanicJinja2.render = render_with_updated_context
 
-
 jinja = SanicJinja2(loader=loader)
 
 
@@ -51,7 +51,7 @@ class App:
 class Config(BaseModel):
     """ Gino Admin Panel settings """
 
-    URL_PREFIX: str = "/admin"
+    route: str = "/admin"
     jinja: SanicJinja2 = None
     app: App = App
     hash_method: Callable = pbkdf2_sha256.encrypt
@@ -62,6 +62,7 @@ class Config(BaseModel):
     max_file_size: int = 10485760
     allowed_file_types: List[str] = ["csv"]
     datetime_str_formats: List[str] = [
+        "%B %d, %Y %I:%M %p",
         "%Y-%m-%dT%H:%M:%S.%f",
         "%Y-%m-%dT%H:%M:%S",
         "%Y-%m-%d %H:%M:%S.%f",
@@ -85,6 +86,21 @@ class Config(BaseModel):
         "name",
         "csv_update_existed",
     ]
+    round_number: float = 3
+    history_model: object = None
+    users_model: object = None
+    history_data_columns: List[str] = []
+    admin_users_data_columns: List[str] = []
+    track_history_endpoints: List[str] = [
+        "app.admin.model_delete",
+        "app.admin.model_delete_all",
+        "app.admin.model_edit_post",
+        "app.admin.model_add",
+        "app.admin.presets_use",
+        "app.admin.init_db_run",
+        "app.admin.file_upload",
+        "app.admin.sql_query_run",
+    ]
 
     @validator("displayable_setting")
     def displayable_setting_cannot_be_changed(cls, value):
