@@ -3,23 +3,29 @@ from copy import deepcopy
 from typing import Dict, List, Text
 
 from gino.ext.sanic import Gino
-from sanic import Blueprint, Sanic, response
+from sanic import Blueprint, Sanic, response, router
 from sanic_jwt import Initialize
 
 from gino_admin import config
 from gino_admin.auth import authenticate
+from gino_admin.history import add_history_model
+# from gino_admin.users import add_users_model
 from gino_admin.routes import rest
 from gino_admin.utils import GinoAdminError, logger, types_map
 
 cfg = config.cfg
 
 
-admin = Blueprint("admin", url_prefix=cfg.URL_PREFIX)
+admin = Blueprint("admin", url_prefix=cfg.route)
+
+STATIC_FOLDER = os.path.join(os.path.dirname(os.path.abspath(__file__)), "static")
+
+admin.static("/static", STATIC_FOLDER)
+admin.static("/favicon.ico", os.path.join(STATIC_FOLDER, "favicon.ico"))
 
 
-admin.static(
-    "/static", os.path.join(os.path.dirname(os.path.abspath(__file__)), "static")
-)
+class HashColumn:
+    pass
 
 
 def extract_column_data(model_id: Text) -> Dict:
@@ -30,7 +36,7 @@ def extract_column_data(model_id: Text) -> Dict:
 
         if _hash in column.name:
             name = column.name.split(_hash)[0]
-            type_ = "HASH"
+            type_ = HashColumn
             hashed_indexes.append(num)
         else:
             name = column.name
@@ -110,12 +116,32 @@ def add_admin_panel(app: Sanic, db: Gino, db_models: List, **config_settings):
                 f"{e}"
             )
 
+    add_history_model(db)
+    # add_users_model(db)
+
     extract_models_metadata(db, db_models)
+    if config_settings.get("route"):
+        old_prefix = admin.url_prefix
+        admin.url_prefix = config_settings["route"]
+        rest.api.url_prefix = str(rest.api.url_prefix).replace(
+            old_prefix, config_settings["route"]
+        )
+
     app.blueprint(admin)
     app.blueprint(rest.api)
-    Initialize(app, authenticate=authenticate, url_prefix="admin/api/auth")
-    Initialize(rest.api, app=app, authenticate=authenticate, auth_mode=True)
+    try:
+        Initialize(app, authenticate=authenticate, url_prefix=f"{cfg.route}/api/auth")
+        Initialize(rest.api, app=app, authenticate=authenticate, auth_mode=True)
+    except router.RouteExists:
+        pass
+    # to avoid re-write app Jinja2
+    if getattr(app, "extensions", None):
+        app_jinja = app.extensions["jinja2"]
+    else:
+        app_jinja = None
     cfg.jinja.init_app(app)
+    if app_jinja:
+        app_jinja.init_app(app)
     cfg.app.config = app.config
 
 
