@@ -2,20 +2,23 @@ import datetime
 import logging
 import os
 import re
-from typing import Any, Dict, List, Text
+import time
+import uuid
+from random import randint
+from typing import Any, Dict, List, Text, Union
 from unicodedata import normalize
-from uuid import uuid4
 
 import aiofiles
 import yaml
 from passlib.hash import pbkdf2_sha256
 
-from gino_admin.config import Config
+from gino_admin import config
 
+cfg = config.cfg
 logger = logging.getLogger("Gino Admin")
 
-salt = uuid4().hex
-cfg = Config
+salt = uuid.uuid4().hex
+
 
 _windows_device_files = (
     "CON",
@@ -39,11 +42,15 @@ types_map = {
     "DECIMAL": float,
     "NUMERIC": float,
     "DATETIME": datetime.datetime,
-    "DATE": datetime.datetime,
+    "DATE": datetime.date,
     "BOOLEAN": bool,
 }
 
 
+class GinoAdminError(Exception):
+    pass
+
+
 def serialize_obj(obj: Any) -> Any:
     """ method to serialise datetime obj """
     if isinstance(obj, (datetime.date, datetime.datetime)):
@@ -132,19 +139,20 @@ def correct_types(params: Dict, columns_data: Dict):
         if "_hash" not in param and not isinstance(
             params[param], columns_data[param]["type"]
         ):
-            if columns_data[param]["type"] is not datetime.datetime:
+            if columns_data[param]["type"] not in [datetime.datetime, datetime.date]:
                 params[param] = columns_data[param]["type"](params[param])
             else:
-                # todo for date
                 params[param] = extract_datetime(params[param])
     for param in to_del:
         del params[param]
+    for column in columns_data:
+        if columns_data[column]["type"] == bool and column not in params:
+            params[column] = False
     return params
 
 
 def extract_date(date_str: Text):
     date_object = datetime.datetime.strptime(date_str, "%m-%d-%y")
-
     return date_object
 
 
@@ -168,6 +176,25 @@ def read_yaml(preset_file):
 
 
 def get_presets():
+    """ return previous loaded, or re-read from disk """
+    if not cfg.presets or (
+        cfg.presets.get("loaded_at")
+        and cfg.presets["loaded_at"] < os.path.getmtime(cfg.presets_folder)
+    ):
+        cfg.presets = {"presets": load_presets(), "loaded_at": time.time()}
+
+    return cfg.presets
+
+
+def get_preset_by_id(preset_id: Text):
+    """ get preset by id """
+    presets = get_presets()["presets"]
+    for preset in presets:
+        if preset_id == preset["id"]:
+            return preset
+
+
+def load_presets():
     """ get presets data from yml configs from presets folder"""
     presets = []
     if not os.path.isdir(cfg.presets_folder):
@@ -187,7 +214,41 @@ def get_presets():
 def get_settings():
     """ gino admin settings '"""
     settings = {}
-    settings_list = ["presets_folder", "composite_csv_settings"]
-    for setting in settings_list:
+    for setting in cfg.displayable_setting:
         settings[setting] = getattr(cfg, setting)
     return settings
+
+
+def generate_new_id(base_key: Text, model_data: Dict) -> Union[Text, int]:
+    key = model_data["key"]
+    if isinstance(base_key, str):
+        new_obj_key = (
+            base_key
+            + f"{'_' if not base_key.endswith('_') else ''}"
+            + uuid.uuid1().hex[5:10]
+        )
+        len_ = model_data["columns_data"][key]["len"]
+        if len_:
+            if new_obj_key[:len_] == base_key:
+                # if we spend all id previous
+                new_obj_key = new_obj_key[
+                    len_ : len_ + len_  # noqa E203
+                ]  # auto format from black
+            else:
+                new_obj_key = new_obj_key[:len_]
+    else:
+        # todo: need to check ints with max size
+        new_obj_key = base_key + randint(0, 10000)
+    return new_obj_key
+
+
+def get_changes(old_obj: Dict, new_obj: Dict):
+    from_ = {}
+    to_ = {}
+    for key, value in new_obj.items():
+        if key not in old_obj:
+            to_[key] = value
+        elif old_obj[key] != value:
+            from_[key] = old_obj[key]
+            to_[key] = value
+    return {"from": from_, "to": to_}
